
# q-expansion of newform 5040.2.a.bi, downloaded from the LMFDB on 27 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5040
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [1], [1], [-2], [-2], [-6], [0], [6], [0], [0], [-6], [-2], [12], [-12], [0], [-4], [-10], [4], [-10], [-14], [0], [0], [-2], [10], [2], [-12], [2], [-14], [-12], [16], [0], [-8], [12], [-4], [16], [10], [-12], [-12], [-22], [-14], [10], [-10], [14], [24], [12], [-4], [16], [-4], [10], [-8], [2], [-18], [-4], [-18], [6], [-10], [-16], [-10], [-20], [4], [-14], [-12], [-20], [14], [24], [-4], [-26], [-10], [14], [-30], [6], [8], [6], [20], [-24], [-16], [34], [36], [14], [12], [-34], [-34], [6], [-20], [30], [24], [38], [42], [-8], [-12], [-12], [16], [-30], [-20], [32], [30], [-6], [8], [6], [4], [0], [4], [-28], [-36], [14], [-28], [-14], [30], [2], [-32], [14], [8], [4], [0], [-4], [-44], [12], [12], [14], [-10], [-34], [42], [46], [-20], [0], [-38], [-16], [28], [38], [28], [42], [-40], [2], [-26], [50], [26], [40], [-14], [8], [-20], [52], [40], [2], [-38], [4], [34], [14], [-8], [14], [38], [-14], [52], [-44], [12], [38], [-16], [-42], [-26], [14], [50], [-20], [32], [40], [-24], [24], [-8], [2], [10], [-6], [-42], [-50], [6], [-54], [-52], [-6], [12], [48], [-36], [-10], [32], [48], [6], [42], [-58], [10], [2], [16], [10], [24], [-54], [-6], [4], [-46], [50], [-54], [-34], [-2], [46], [-4], [12], [28], [46], [-50], [48], [-54], [4], [34], [-16], [-24], [10], [46], [24], [-48], [12], [18], [-16], [2], [62], [36], [10], [56], [24], [-24], [40], [-54], [-10], [66], [68], [-22], [-6], [48], [48], [-24], [28], [-16], [14], [40], [-62], [0], [6], [-52], [-32], [42], [54], [-20], [-44], [12], [4], [6], [-62], [-22], [-14], [14], [-6], [78], [-26], [68], [-6], [6], [24], [-66], [-30], [10], [-54], [-20], [44], [-70], [-60], [-44], [-18], [-20], [-38], [56], [58], [20], [66], [-54], [14], [76], [8], [-64], [8], [14], [-24], [-42], [14], [-24], [-24], [54], [60], [-8], [24], [-28], [62], [-54], [20], [14], [-84], [-48], [-2], [28], [16], [-86], [60], [22], [-82], [-54], [30], [58], [-72], [0], [-68], [-68], [54], [70], [64], [-70], [-46], [-36], [-8], [-10], [-62], [40], [28], [-34], [-84], [-4], [26], [20], [-14], [-68], [-56], [-52], [84], [60], [46], [-10], [-42], [44], [70], [-48], [-58], [-48], [36], [6], [2], [52], [-8], [18], [-40], [-38], [-76], [16], [-74], [-14], [-48], [-24], [-20], [74], [70], [-90], [66], [22], [48], [-10], [70], [-56], [30], [-38], [16], [-6], [12], [-88], [62], [-96], [82], [-66], [66], [-54], [-54], [84], [-64], [-92], [44], [-8], [40], [62], [-76], [-52], [2], [22], [-90], [44], [-10], [14], [-20], [26], [-92], [-32], [-90], [-4], [16], [62], [-18], [68], [-2], [0], [-56], [76], [46], [-52], [-90], [36], [-38], [-42], [18], [-84], [-14], [-60], [-56], [-102], [-86], [0], [6], [-70], [38], [12], [-52], [108]]
